@@ -24,6 +24,7 @@ use std::{env, fs::File};
 mod ansi;
 mod commit_line;
 mod constants;
+mod hunk_header;
 mod line_collector;
 mod refiner;
 mod token_collector;
@@ -92,6 +93,26 @@ struct Options {
     please_panic: bool,
 }
 
+/// Consume the line
+///
+/// If that fails, print an error message and exit with an error code.
+fn consume_line_or_exit(
+    line_collector: &mut LineCollector,
+    line_number: usize,
+    line: &mut Vec<u8>,
+) {
+    if let Some(error_message) = line_collector.consume_line(line) {
+        eprintln!(
+            "ERROR on line {}: {}\n         Line {}: {}",
+            line_number,
+            error_message,
+            line_number,
+            String::from_utf8_lossy(line),
+        );
+        exit(1);
+    }
+}
+
 fn highlight_diff<W: io::Write + Send + 'static>(input: &mut dyn io::Read, output: W) {
     let mut line_collector = LineCollector::new(output);
 
@@ -99,6 +120,7 @@ fn highlight_diff<W: io::Write + Send + 'static>(input: &mut dyn io::Read, outpu
     // strings while handling invalid UTF-8 without crashing
     let mut line: Vec<u8> = Vec::new();
     let mut buf: [u8; 16384] = [0; 16384];
+    let mut line_number = 1usize;
     loop {
         let result = input.read(&mut buf);
         if result.is_err() {
@@ -110,7 +132,7 @@ fn highlight_diff<W: io::Write + Send + 'static>(input: &mut dyn io::Read, outpu
             // End of stream
             if !line.is_empty() {
                 // Stuff found on the last line without a trailing newline
-                line_collector.consume_line(&mut line);
+                consume_line_or_exit(&mut line_collector, line_number, &mut line);
             }
             break;
         }
@@ -128,8 +150,9 @@ fn highlight_diff<W: io::Write + Send + 'static>(input: &mut dyn io::Read, outpu
             }
 
             // Line finished, consume it!
-            line_collector.consume_line(&mut line);
+            consume_line_or_exit(&mut line_collector, line_number, &mut line);
             line.clear();
+            line_number += 1;
             continue;
         }
     }
@@ -372,7 +395,7 @@ fn main() {
 
 #[cfg(test)]
 mod tests {
-    use crate::constants::*;
+    use crate::{constants::*, hunk_header::HUNK_HEADER};
 
     use super::*;
     use std::{fs, path::PathBuf};
@@ -386,10 +409,13 @@ mod tests {
 
     #[test]
     fn test_trailing_newline_context() {
-        let mut input = "+bepa\n apa\n\\ No newline at end of file\n".as_bytes();
+        let mut input = "@@ -1,1 +1,2 @@\n+bepa\n apa\n\\ No newline at end of file\n".as_bytes();
 
         let expected = format!(
-            "{}\n{}\n{}\\ No newline at end of file{}\n",
+            "{}{}{}\n{}\n{}\n{}\\ No newline at end of file{}\n",
+            HUNK_HEADER,
+            "@@ -1,1 +1,2 @@",
+            NORMAL,
             new("+bepa"),
             " apa",
             NO_EOF_NEWLINE_COLOR,
